#include <m_pd.h>
#include <cwiid.h>

#include "wiiIR.h"

void wiiIR_input(t_wiiIR* x, t_symbol *s, int argc, t_atom *argv) {
	if(x->count_active) {
		x->inc_count = atom_getint(argv);
		if ((x->inc_count >= 0) && (x->inc_count < 4)) {
			x->blob[x->inc_count]++;
		}
	}
}

void wiiIR_start(t_wiiIR* x) {
	x->count_active = 1;
	t_int i;
	for (i = 0; i < 4; i++) {
		x->blob[i] = 0;
	}
	clock_delay(x->counter_clock, x->frequency);
}

void wiiIR_stop(t_wiiIR* x) {
	x->count_active = 0;
}

void wiiIR_count(t_wiiIR* x) {
	t_int counter = 0;
	t_int i;
	for (i = 0; i < 4; i++) {
		if (x->blob[i] > (x->blob[0] / 2)) {
			counter++;
		}
		x->blob[i] = 0;
	}
	outlet_float(x->outlet_count, counter);
	if (x->count_active) {
		clock_delay(x->counter_clock, x->frequency);
	}
}

void *wiiIR_new(t_symbol *s, int argc, t_atom *argv) {
	t_wiiIR *x = (t_wiiIR *)pd_new(wiiIR_class);
	x->frequency = 1;
	x->count_active = 0;
	if (argc) {
		t_symbol *temp;
		temp = atom_getsymbol(argv);
		if (temp == gensym("count")) {
			x->outlet_count = outlet_new(&x->x_obj, &s_float);
			x->counter_clock = clock_new(x, (t_method)wiiIR_count);
			if (argc > 0) {
				t_int i = atom_getint(argv+1);
				if (i > 0) {
					x->frequency = i;
				}
			}
			clock_delay(x->counter_clock, x->frequency);
		}
	}
	return (void *)x;
}

void wiiIR_free(t_wiiIR *x) {
	clock_free(x->counter_clock);
}

void wiiIR_setup(void) {
	wiiIR_class = class_new(gensym("wiiIR"),
		(t_newmethod)wiiIR_new,
		(t_method)wiiIR_free, sizeof(t_wiiIR),
		CLASS_DEFAULT, A_GIMME, 0);
	class_addlist(wiiIR_class, (t_method)wiiIR_input);
	class_addmethod(wiiIR_class, (t_method)wiiIR_start, gensym("start"), 0);
	class_addmethod(wiiIR_class, (t_method)wiiIR_stop, gensym("stop"), 0);
}
