// A small external to interpret the Wiimote button output
// Not really need, but could be handy
//
// Usage: Just add the buttons you want as an argument, in the order
//        you want to have the outlets. Possible names are:
//        lcross, rcross, dcross, ucross
//        a, minus, plus, home
//        one, two, back
//        
// ===================================================================
// wiibuttons external
// Part of the Wiilib library for Puredata
// Written by Yvan Vander Sanden (Jan 2008), http://youngmusic.org
//
// Requires the CWiid library (version 0.6.00) by L. Donnie Smith
//
// ===================================================================
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// ===================================================================
//

#include <m_pd.h>
#include <cwiid.h>

#include "wiibuttons.h"

void wiibuttons_input(t_wiibuttons *x, t_symbol *s, int argc, t_atom *argv) {
	t_int a = atom_getint(argv);
	t_int b = atom_getint(argv+1);
	t_int v;
	if(x->lcross_exists) {
		v = a & 0x1;
		if(x->b_lcross != v) {
			x->b_lcross = v;
			outlet_float(x->outlet_lcross, x->b_lcross);
		}
	}
	if(x->rcross_exists) {
		v = (a & 0x2) >> 1;
		if(x->b_rcross != v) {
			x->b_rcross = v;
			outlet_float(x->outlet_rcross, x->b_rcross);
		}
	}
	if(x->dcross_exists) {
		v = (a & 0x4) >> 2;
		if(x->b_dcross != v) {
			x->b_dcross = v;
			outlet_float(x->outlet_dcross, x->b_dcross);
		}
	}
        if(x->ucross_exists) {
                v = (a & 0x8) >> 3;
                if(x->b_ucross != v) {
                        x->b_ucross = v;
                        outlet_float(x->outlet_ucross, x->b_ucross);
                }
        }
        if(x->plus_exists) {
                v = (a & 0x10) >> 4;
                if(x->b_plus != v) {
                        x->b_plus = v;
                        outlet_float(x->outlet_plus, x->b_plus);
                }
        }
	if(x->two_exists) {
		v = b & 0x1;
		if(x->b_two != v) {
			x->b_two = v;
			outlet_float(x->outlet_two, x->b_two);
		}
	}
	if(x->one_exists) {
		v = (b & 0x2) >> 1;
		if(x->b_one != v) {
			x->b_one = v;
			outlet_float(x->outlet_one, x->b_one);
		}
	}
	if(x->back_exists) {
		v = (b & 0x4) >> 2;
		if(x->b_back != v) {
			x->b_back = v;
			outlet_float(x->outlet_back, x->b_back);
		}
	}
        if(x->a_exists) {
                v = (b & 0x8) >> 3;
                if(x->b_a != v) {
                        x->b_a = v;
                        outlet_float(x->outlet_a, x->b_a);
                }
        }
        if(x->minus_exists) {
                v = (b & 0x10) >> 4;
                if(x->b_minus != v) {
                        x->b_minus = v;
                        outlet_float(x->outlet_minus, x->b_minus);
                }
        }	
        if(x->home_exists) {
                v = (b & 0x80) >> 5;
                if(x->b_home != v) {
                        x->b_home = v;
                        outlet_float(x->outlet_home, x->b_home);
                }
        }
																	
}

void *wiibuttons_new(t_symbol *s, int argc, t_atom *argv) {
	t_wiibuttons *x = (t_wiibuttons *)pd_new(wiibuttons_class);
	x->lcross_exists = 0;
	x->b_lcross = 0;
	x->rcross_exists = 0;
	x->b_rcross = 0;
	x->dcross_exists = 0;
	x->b_dcross = 0;
	x->ucross_exists = 0;
	x->b_ucross = 0;
	x->a_exists = 0;
	x->b_a = 0;
	x->minus_exists = 0;
	x->b_minus = 0;
	x->home_exists = 0;
	x->b_home = 0;
	x->plus_exists = 0;
	x->b_plus = 0;
	x->one_exists = 0;
	x->b_one = 0;
	x->two_exists = 0;
	x->b_two = 0;
	x->back_exists = 0;
	x->b_back = 0;
	int i;
	for (i = 0; i < argc; i++) {
		t_symbol *temp;
		temp = atom_getsymbol(argv+i);
		if (temp == gensym("lcross")) {
			x->outlet_lcross = outlet_new(&x->x_obj, &s_float);
			x->lcross_exists = 1;
		}
                if (temp == gensym("rcross")) {
                        x->outlet_rcross = outlet_new(&x->x_obj, &s_float);
                        x->rcross_exists = 1;
                }
                if (temp == gensym("dcross")) {
                        x->outlet_dcross = outlet_new(&x->x_obj, &s_float);
                        x->dcross_exists = 1;
                }	
                if (temp == gensym("ucross")) {
                        x->outlet_ucross = outlet_new(&x->x_obj, &s_float);
                        x->ucross_exists = 1;
                }			
                if (temp == gensym("a")) {
                        x->outlet_a = outlet_new(&x->x_obj, &s_float);
                        x->a_exists = 1;
                }
                if (temp == gensym("minus")) {
                        x->outlet_minus = outlet_new(&x->x_obj, &s_float);
                        x->minus_exists = 1;
                }
                if (temp == gensym("home")) {
                        x->outlet_home = outlet_new(&x->x_obj, &s_float);
                        x->home_exists = 1;
                }
                if (temp == gensym("plus")) {
                        x->outlet_plus = outlet_new(&x->x_obj, &s_float);
                        x->plus_exists = 1;
                }
		if (temp == gensym("one")) {
			x->outlet_one = outlet_new(&x->x_obj, &s_float);
			x->one_exists = 1;
		} 
                if (temp == gensym("two")) {
                        x->outlet_two = outlet_new(&x->x_obj, &s_float);
                        x->two_exists = 1;
                }
                if (temp == gensym("back")) {
                       x->outlet_back = outlet_new(&x->x_obj, &s_float);
                       x->back_exists = 1;
		}													 
	}
	return (void *)x;
}

void wiibuttons_setup(void) {
	wiibuttons_class = class_new(gensym("wiibuttons"),
		(t_newmethod)wiibuttons_new,
		0, sizeof(t_wiibuttons),
		CLASS_DEFAULT, A_GIMME, 0);
	class_addlist(wiibuttons_class, (t_method)wiibuttons_input);
}
