// A the main external for wiimote. This one does all the work.
// For the most part this is a rewrite of the original wiimote
// external written by Mike Wozniewki.
//
// I tried to get Mike's approvement before using his code, but there
// was something wrong with his mailserver. hopefully he doesn't mind.
// At any rate everything is GPL, so is should not really matter.
// 
// Usage is explained in the included help patch.
//
// ===================================================================
// WiiAcc external
// Part of the Wiilib library for Puredata
// Written by Yvan Vander Sanden (Jan 2008), http://youngmusic.org
// Parts written by Mike Wozniewki (Feb 2007), www.mikewoz.com
//
// Requires the CWiid library (version 0.6.00) by L. Donnie Smith
//
// ===================================================================
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
// ===================================================================
//
//

#include <m_pd.h>
#include <bluetooth/bluetooth.h>
#include <cwiid.h>
#include <glib.h>

#include "wiimote.h"

// some handy defined macros for bitwise operations

#define toggle_bit(bf,b) \
	(bf) = ((bf) & b) \
		? ((bf) & ~(b)) \
		: ((bf) | (b))

#define turn_on(bf,b) \
	(bf) = bf | b

#define turn_off(bf,b) \
	(bf) = ((bf) & ~(b))	


// Each wiimote calls this function when it sends information
// to our program.
//
void cwiid_callback(cwiid_wiimote_t *wiimote, int mesg_count,
	union cwiid_mesg mesg[], struct timespec *timestamp) {
	int i;
	t_wiimote *x;
	int found;
	// find the wiimote object for the controller that sends
	// the message. This way, our object is called x, and can
	// be passed just like in any other function.
	found = -1;
	for (i = 0; i < list->len; i++) {
		x = g_array_index(list, t_wiimote*, i);
		if (x->wiimote == wiimote) {
			found = i;
		}
	}
	if (found == -1) {
		post("callback: wiimote not found");
	} else {
		x = g_array_index(list, t_wiimote*, found);
		// now handle all messages. They are in an array, so we just have to pass
		// each type to the correlating function
		//
		for (i = 0; i < mesg_count; i++) {
			switch (mesg[i].type) {
			case CWIID_MESG_STATUS:
				post("%s Wiimote Status Report: battery is at %d%", x->ID->s_name, 
					(int) (100.0 * mesg[i].status_mesg.battery / CWIID_BATTERY_MAX));
				switch(mesg[i].status_mesg.ext_type) {
				case CWIID_EXT_NONE:
					post("%s Wiimote extension Report: none", x->ID->s_name);
					break;
				case CWIID_EXT_NUNCHUK:
					post("%s Wiimote extension Report: Nunchuk", x->ID->s_name);
					break;
				case CWIID_EXT_CLASSIC:
					post("%s Wiimote extension Report: Classic Controller", x->ID->s_name);
					break;
				default:
					post("%s Wiimote extension Report: Unknown", x->ID->s_name);
					break;
				}
				break;
			case CWIID_MESG_BTN:
				wiimote_button_report(x, &mesg[i].btn_mesg);
				break;
			case CWIID_MESG_ACC:
				wiimote_acc_report(x, &mesg[i].acc_mesg);
				break;
			case CWIID_MESG_IR:
				wiimote_ir_report(x, &mesg[i].ir_mesg);
				break;
			case CWIID_MESG_NUNCHUK:
				wiimote_nunchuk_report(x, &mesg[i].nunchuk_mesg);
				break;
			case CWIID_MESG_CLASSIC:
				wiimote_classic_report(x, &mesg[i].classic_mesg);
				break;
			case CWIID_MESG_ERROR:
				if (cwiid_close(wiimote)) {
					post("Error on wiimote disconnect");
				}
				break;
			default:
				post("Unknown Report");
				break;
			}
		}
	}
}

// the debug message gives us some basic information in the console
// You'll see the connection status, the extension status and the 
// calibration values for the remote and the nunchuk
//
void wiimote_debug(t_wiimote *x) {
	post("\n=====================");
	if (x->connected) post("%s Wiimote is connected", x->ID->s_name);
	else post("%s Wiimote is NOT connected.", x->ID->s_name);
	unsigned char v;
	v = (x->receive & CWIID_RPT_BTN) >> 1;
	if(v) post("    buttons: ON");
	else post("    buttons: OFF");
	v = (x->receive & CWIID_RPT_ACC) >> 2;
	if(v) post("    acceleration: ON");
	else post("    acceleration: OFF");
	v = (x->receive & CWIID_RPT_IR) >> 3;
	if(v) post("    IR: ON");
	else post("    IR: OFF");
	v = (x->receive & CWIID_RPT_NUNCHUK) >> 4;
	if(v) post("    Nunchuk: ON");
	else post("    Nunchuk: OFF");
	v = (x->receive & CWIID_RPT_CLASSIC) >> 5;
	if(v) post("    Classic: ON");
	else post("    Classic: OFF");
	post("");
	post("Accelerometer calibration: zero=(%d, %d, %d) one=(%d,%d,%d)", 
		x->cal.zero[CWIID_X], x->cal.zero[CWIID_Y], x->cal.zero[CWIID_Z],
		x->cal.one[CWIID_X], x->cal.one[CWIID_Y], x->cal.one[CWIID_Z]);
	post("Nunchuk calibration: zero=(%d, %d, %d) one=(%d,%d,%d)",
                x->nc_cal.zero[CWIID_X], x->nc_cal.zero[CWIID_Y], x->nc_cal.zero[CWIID_Z],
	        x->nc_cal.one[CWIID_X], x->nc_cal.one[CWIID_Y], x->nc_cal.one[CWIID_Z]);
}

// We could also outpubt the button message directly, but i'll keep it this way
// to stay compatible with the original external.
// Now there are two atoms who have to be unpacked. This can be a nuisance because
// both sides of the unpacked output will trigger, even if the second one did not change
// anything.
// I have written another object, wiibuttons to solve this problem. (Well it's not
// really a problem, but output is easier to control this way.)
//
void wiimote_button_report(t_wiimote *x, struct cwiid_btn_mesg *mesg) {
	SETFLOAT(x->btn_atoms+0, (mesg->buttons & 0xFF00)>>8);
	SETFLOAT(x->btn_atoms+1, mesg->buttons & 0x00FF);
	outlet_anything(x->outlet_btn, &s_list, 2, x->btn_atoms);
}

// For acceleration, all that matters are these three values. But 
// i've also made an external called wiiAcc. that can be used to produce
// roll, pitch or total acceleration very easy.
//
void wiimote_acc_report(t_wiimote *x, struct cwiid_acc_mesg *mesg) {
	double a_x, a_y, a_z;
	a_x = ((double)mesg->acc[CWIID_X] - x->cal.zero[CWIID_X]) / (x->cal.one[CWIID_X] - x->cal.zero[CWIID_X]);
	a_y = ((double)mesg->acc[CWIID_Y] - x->cal.zero[CWIID_Y]) / (x->cal.one[CWIID_Y] - x->cal.zero[CWIID_Y]);
	a_z = ((double)mesg->acc[CWIID_Z] - x->cal.zero[CWIID_Z]) / (x->cal.one[CWIID_Z] - x->cal.zero[CWIID_Z]);

	SETFLOAT(x->acc_atoms+0, a_x);
	SETFLOAT(x->acc_atoms+1, a_y);
	SETFLOAT(x->acc_atoms+2, a_z);
	outlet_anything(x->outlet_acc, &s_list, 3, x->acc_atoms);
}

// Apparently libcwiid only supports a maximum of 4 IR sources.
// I guess these are limitations of the wii itsself, but i don't
// really know.
//
void wiimote_ir_report(t_wiimote *x, struct cwiid_ir_mesg *mesg) {
	int i;
	for (i=0; i < CWIID_IR_SRC_COUNT; i++) {
		// Sources are outputted one by one, but only if they
		// contain valid information. (This means the source
		// is actually detected).
		//
		if (mesg->src[i].valid) {
			SETFLOAT(x->ir_atoms+0, i);
			SETFLOAT(x->ir_atoms+1, mesg->src[i].pos[CWIID_X]);
			SETFLOAT(x->ir_atoms+2, mesg->src[i].pos[CWIID_Y]);
			SETFLOAT(x->ir_atoms+3, mesg->src[i].size);
			outlet_anything(x->outlet_ir, &s_list, 4, x->ir_atoms);
		}
	}
}

// I don't have a nunchuk myself, so this code is untested. It should be ok though,
// since it hardly differs from Mike's original code.
//
void wiimote_nunchuk_report(t_wiimote *x, struct cwiid_nunchuk_mesg *mesg) {
	double a_x, a_y, a_z;
	
// nunchuk calibration is not supported yet. Thanks to Mario Kishalmi for pointing
// that out.
//	a_x = ((double)mesg->acc[CWIID_X] - x->nc_cal.zero[CWIID_X]) / (x->nc_cal.one[CWIID_X] - x->nc_cal.zero[CWIID_X]);
//	a_y = ((double)mesg->acc[CWIID_Y] - x->nc_cal.zero[CWIID_Y]) / (x->nc_cal.one[CWIID_Y] - x->nc_cal.zero[CWIID_Y]);
//	a_z = ((double)mesg->acc[CWIID_Z] - x->nc_cal.zero[CWIID_Z]) / (x->nc_cal.one[CWIID_Z] - x->nc_cal.zero[CWIID_Z]);

	// temporary replacement for the previous
	a_x = (double)mesg->acc[CWIID_X];
	a_y = (double)mesg->acc[CWIID_Y];
	a_z = (double)mesg->acc[CWIID_Z];
	// endof temporary replacement
	

	outlet_float(x->outlet_nc_btn, mesg->buttons);
	
	// output nunchuk acceleration
	//
	SETFLOAT(x->nc_acc_atoms+0, a_x);
	SETFLOAT(x->nc_acc_atoms+1, a_y);
	SETFLOAT(x->nc_acc_atoms+2, a_z);
        outlet_anything(x->outlet_nc_acc, &s_list, 3, x->nc_acc_atoms);
	
	// output the nunchuk stick x and y axis
	//
        SETFLOAT(x->nc_stick_atoms+0, mesg->stick[CWIID_X]);
        SETFLOAT(x->nc_stick_atoms+1, mesg->stick[CWIID_Y]);
        outlet_anything(x->outlet_nc_stick, &s_list, 2, x->nc_stick_atoms);
}					

// This is even more experimental. The original external did not have
// classic support, and I do not own one. however, it should be similar to
// the nunchuk. So it's an educated guess this code will work. To keep the
// amount of outlets within reason, i'm using the same outlets as for the
// nunchuk (one extra) since they cannot be connected both at the same time.
//
void wiimote_classic_report(t_wiimote *x, struct cwiid_classic_mesg *mesg) {

	// Output all buttons at once. You will need bitwise 
	// operations to make sense of them. Ideally, an external
	// like wiibuttons would be very usefull here.
	//
	outlet_float(x->outlet_nc_btn, mesg->buttons);

	// output left and right value. I don't have a clue how they
	// are generated...
	//
	SETFLOAT(x->nc_acc_atoms+0, mesg->l);
	SETFLOAT(x->nc_acc_atoms+1, mesg->r);
	outlet_anything(x->outlet_nc_acc, &s_list, 2, x->nc_acc_atoms);

	// output x and y axis for the first stick
	SETFLOAT(x->nc_stick_atoms+0, mesg->l_stick[CWIID_X]);
	SETFLOAT(x->nc_stick_atoms+1, mesg->l_stick[CWIID_Y]);
	outlet_anything(x->outlet_nc_stick, &s_list, 2, x->nc_stick_atoms);

	// output x and y axis for the second stick
	SETFLOAT(x->classic_stick_atoms+0, mesg->r_stick[CWIID_X]);
	SETFLOAT(x->classic_stick_atoms+1, mesg->r_stick[CWIID_X]);
	outlet_anything(x->outlet_classic_stick, &s_list, 2, x->classic_stick_atoms);
}

// set all LEDs at once. The four LEDs are bitwise toggled.
//
void wiimote_setLED(t_wiimote *x, t_floatarg f) {
	if((x->connected) && (f >= 0) && (f < 16)) {
		if(cwiid_command(x->wiimote, CWIID_CMD_LED, f)) {
			post("%s Wiimote error: problem setting LED.", x->ID->s_name);
		} else {
			// state is kept so we can also toggle one LED
			// ourselves
			//
			x->led_state = f;
		}
	}
}

// To set a single led. The value used in x->led_state is set
// by the functions who call this.
//
void set_led_state(t_wiimote *x) {
	if (x->connected) {
		if (cwiid_set_led(x->wiimote, x->led_state)) {
			post("%s Wiimote: Error setting LEDs", x->ID->s_name);
		}
	} else {
		post("%s Wiimote: not connected", x->ID->s_name);
	}
}

// The wiimote does not report all kinds of values when not asked for it
// Every type of information has to be asked first. All types are written to
// the x->receive var and passed to the wiimote here. This function can be
// called by its own (if you want to completely overwrite x->receive)
// or it can be called with f = -1 by other functions if you want to 
// use x->receive as it is.
//
void wiimote_setReportMode(t_wiimote *x, t_floatarg f) {
	if (f >= 0) {
		x->receive = f;
	}
	if (x->connected) {
		if (cwiid_set_rpt_mode(x->wiimote, x->receive)) {
			post("%s Wiimote: Error setting report mode.", x->ID->s_name);
		}
	} else {
		post("%s Wiimote: not connected", x->ID->s_name);
	}
}


// set the individual LED's. A bit redundant, but very handy.
void wiimote_LED1(t_wiimote *x) {
	toggle_bit(x->led_state, CWIID_LED1_ON);
	set_led_state(x);
}

void wiimote_LED2(t_wiimote *x) {
        toggle_bit(x->led_state, CWIID_LED2_ON);
        set_led_state(x);
}

void wiimote_LED3(t_wiimote *x) {
        toggle_bit(x->led_state, CWIID_LED3_ON);
        set_led_state(x);
}

void wiimote_LED4(t_wiimote *x) {
        toggle_bit(x->led_state, CWIID_LED4_ON);
        set_led_state(x);
}

// does what it says
//
void wiimote_rumble(t_wiimote *x, t_floatarg f) {
	if (x->connected) {
		if ((f == 0) || (f == 1)) {
			cwiid_set_rumble(x->wiimote, f);
		}
	} else {
		post("%s Wiimote: note connected", x->ID->s_name);
	}
}

// Get a small status report. Output is generated in the callback function
// directly.
//
void wiimote_status(t_wiimote *x) {
	turn_on(x->receive, CWIID_RPT_STATUS);
	wiimote_setReportMode(x, -1);
	if (cwiid_request_status(x->wiimote)) {
		post("%s Wiimote: error requesting status message", x->ID->s_name);
	}
	// should be turned of later, but the status report is never
	// received when we turn status off here. Hence it's commented
	// out.
	//
//	turn_off(x->receive, CWIID_RPT_STATUS);
//	wiimote_setReportMode(x, -1);
}

// enable the button bit in receive. If not, button
// signals will not be sent by the wii remote.
//
void wiimote_buttons(t_wiimote *x, t_floatarg f) {
	if (f == 0) {
		// turn_off and turn_on are macros
		// defined at the beginning of this file
		//
		turn_off(x->receive, CWIID_RPT_BTN);
		wiimote_setReportMode(x, -1);
	}
	if (f == 1) {
		turn_on(x->receive, CWIID_RPT_BTN);
		wiimote_setReportMode(x, -1);
	}
}

// same here...
//
void wiimote_acceleration(t_wiimote *x, t_floatarg f) {
	if (f == 0) {
		turn_off(x->receive, CWIID_RPT_ACC);
		 wiimote_setReportMode(x, -1);
	}
	if (f == 1) {
		turn_on(x->receive, CWIID_RPT_ACC);
		 wiimote_setReportMode(x, -1);
	}
}

// and here...
//
void wiimote_IR(t_wiimote *x, t_floatarg f) {
        if (f == 0) {
                turn_off(x->receive, CWIID_RPT_IR);
                 wiimote_setReportMode(x, -1);
        }
        if (f == 1) {
                turn_on(x->receive, CWIID_RPT_IR);
                 wiimote_setReportMode(x, -1);
       }
}

// here too...
//
void wiimote_nunchuk(t_wiimote *x, t_floatarg f) {
        if (f == 0) {
                turn_off(x->receive, CWIID_RPT_NUNCHUK);
                 wiimote_setReportMode(x, -1);
        }
        if (f == 1) {
                turn_on(x->receive, CWIID_RPT_NUNCHUK);
                 wiimote_setReportMode(x, -1);
        }
}

// last one...
//
void wiimote_classic(t_wiimote *x, t_floatarg f) {
        if (f == 0) {
                turn_off(x->receive, CWIID_RPT_CLASSIC);
                 wiimote_setReportMode(x, -1);
        }
        if (f == 1) {
                turn_on(x->receive, CWIID_RPT_CLASSIC);
                 wiimote_setReportMode(x, -1);
        }
}

// oh wait, another!
// Well this one is redundant too, but it exists in the library too.
// It actually turns on status report for the nunchuk as well as
// the classic extension.
//
void wiimote_extension(t_wiimote *x, t_floatarg f) {
        if (f == 0) {
                turn_off(x->receive, CWIID_RPT_EXT);
                 wiimote_setReportMode(x, -1);
        }
        if (f == 1) {
                turn_on(x->receive, CWIID_RPT_EXT);
                 wiimote_setReportMode(x, -1);
        }
}

// Discover a wii on the bluetooth network.
//
void wiimote_discover(t_wiimote *x) {
	 post("%s Wiimote: put into discover mode by pressing buttons 1 and 2 simultaneously.", x->ID->s_name);
	 wiimote_connect(x, gensym("NULL"));
	 if (!(x->connected)) {
		post("Error: could not find any wiimotes. Please ensure that bluetooth is enabled, and that the 'hcitool scan' command lists your Nintendo device.");
	 }
}


// The following function attempts to connect to a wiimote at a
// specific address, provided as an argument. eg, 00:19:1D:70:CE:72
// This address can be discovered by running the following command
// in a console:
//   hcitool scan | grep Nintendo
//
void wiimote_connect(t_wiimote *x, t_symbol *addr) {
	if (x->connected == 0) {
		bdaddr_t bdaddr;
		// determine address
		if (addr == gensym("NULL")) {
			// if the function is called by discover
			bdaddr = (bdaddr_t) *BDADDR_ANY;
		}
		// if an address is provided
		else str2ba(addr->s_name, &bdaddr);
		
		// try to get a connection
		if ((x->wiimote = cwiid_open(&bdaddr, CWIID_FLAG_MESG_IFC)) == NULL) {
			post("%s Wiimote: unable to connect", x->ID->s_name);
		} else {
			post("%s Wiimote: connected.", x->ID->s_name);
			x->connected = 1;
			// the callback function must be able to get the pd object
			// but it cannot be passed as an argument. Therefore we make
			// an array with pointers to every object. A global variable
			// is used to make sure the array is only initialized if it
			// does not exist.
			if (setup == 0) {
				list = g_array_new(TRUE, FALSE, sizeof(t_wiimote*));
			}
						
			// add new object to object list
			list = g_array_append_val(list, x);
			// A global variable keeps track of the number of connections
			// to make sure the GArray is created and freed if needed. It
			// is initiallly set to zero in wiilib.c
			setup++;

			// set up the callback function
			//
			if (cwiid_set_mesg_callback(x->wiimote, cwiid_callback)) {
				post("%s Wiimote: Unable to set message callback.", x->ID->s_name);
			} else {
				// calibrate the accelerometer
				//
				if (cwiid_get_acc_cal(x->wiimote, CWIID_EXT_NONE, &x->cal)) {
					post("%s Wiimote: Unable to retrieve accelerometer calibration", x->ID->s_name);
				} 
			}
		}
	}
	// enable button report by default since the original extension does this too
	//
	wiimote_buttons(x, 1);
}


void *wiimote_new(t_symbol *s, int argc, t_atom *argv) {

	t_wiimote *x = (t_wiimote *)pd_new(wiimote_class);
	x->connected = 0;
	x->led_state = 0;

	// the argument is only used to give each wii a meaningfull name 
	// in post messages.
	//
	switch (argc) {
	default:
	case 1:
		x->ID = atom_getsymbol(argv);
		break;
	case 0:
		x->ID = gensym("default");
	}

	// create outlets
	x->outlet_btn = outlet_new(&x->x_obj, &s_list);
        x->outlet_acc = outlet_new(&x->x_obj, &s_list);
        x->outlet_ir = outlet_new(&x->x_obj, &s_list);
        x->outlet_nc_btn = outlet_new(&x->x_obj, &s_float);
        x->outlet_nc_acc = outlet_new(&x->x_obj, &s_list);
        x->outlet_nc_stick = outlet_new(&x->x_obj, &s_list);
	x->outlet_classic_stick = outlet_new(&x->x_obj, &s_list);

	return(void *)x;
}

void wiimote_disconnect(t_wiimote* x) {
        if (x->connected) {
		
		// Pull the pointer of the current wii out
		// of the global list which is used by the
		// callback function
		//
		t_wiimote* y;
		int i;
	        for (i = 0; i < list->len; i++) {
	            y = g_array_index(list, t_wiimote*, i);
	            if (x->wiimote == y->wiimote) {
	                g_array_remove_index(list, i);
			setup--;
	            }
	        }
		// If no more connected wii's are left,
		// memory must be freed
		//
		if (setup == 0) {
			g_array_free(list, FALSE);
		}
		if (cwiid_close(x->wiimote)) {
			post("%s Wiimote error: problems when disconnecting.", x->ID->s_name);
		} else {
			x->connected = 0;
			x->led_state = 0;
			post("%s Wiimote disconnected.", x->ID->s_name);
		}
	}
}

void wiimote_free(t_wiimote* x) {
	wiimote_disconnect(x);
}

void wiimote_setup(void) {
	wiimote_class = class_new(gensym("wiimote"),
		(t_newmethod)wiimote_new,
		(t_method)wiimote_free, sizeof(t_wiimote),
		CLASS_DEFAULT, A_GIMME, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_debug, gensym("debug"), 0);
	class_addmethod(wiimote_class, (t_method)wiimote_connect, gensym("connect"), A_SYMBOL, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_disconnect, gensym("disconnect"), 0);
	class_addmethod(wiimote_class, (t_method)wiimote_discover, gensym("discover"), 0);
	class_addmethod(wiimote_class, (t_method)wiimote_setReportMode, gensym("setReportMode"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_setLED, gensym("setLED"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_LED1, gensym("LED1"), 0);
	class_addmethod(wiimote_class, (t_method)wiimote_LED2, gensym("LED2"), 0);
	class_addmethod(wiimote_class, (t_method)wiimote_LED3, gensym("LED3"), 0);
	class_addmethod(wiimote_class, (t_method)wiimote_LED4, gensym("LED4"), 0);	
	class_addmethod(wiimote_class, (t_method)wiimote_rumble, gensym("setRumble"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_buttons, gensym("reportButtons"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_acceleration, gensym("reportAcceleration"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_IR, gensym("reportIR"), A_DEFFLOAT, 0);

	// support the spelling mistake in the original version :-)
	class_addmethod(wiimote_class, (t_method)wiimote_nunchuk, gensym("reportNunchuck"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_nunchuk, gensym("reportNunchuk"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_classic, gensym("reportClassic"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_extension, gensym("reportExtension"), A_DEFFLOAT, 0);
	class_addmethod(wiimote_class, (t_method)wiimote_status, gensym("reportStatus"), 0);
}

