static t_class *wiimote_class;

typedef struct _wiimote {
        t_object x_obj;

	// pointer to a connected wii remote
	cwiid_wiimote_t *wiimote;

	// these structures are declared in the cwiid.h
	struct acc_cal cal;
	struct acc_cal nc_cal;
	t_int connected;
	unsigned char led_state;

	// keeps which values are sent by the wii
	unsigned char receive;

	// only a name
	t_symbol *ID;

	// it's faster to declare these here than
	// declare them again every time a message is received
	t_atom btn_atoms[2];
	t_atom acc_atoms[3];
        t_atom ir_atoms[4];
        t_atom nc_btn_atoms[2];
        t_atom nc_acc_atoms[3];
        t_atom nc_stick_atoms[2];
	t_atom classic_stick_atoms[2];
					
	// outlets
	t_outlet *outlet_btn;
	t_outlet *outlet_acc;
        t_outlet *outlet_ir;
        t_outlet *outlet_nc_btn; // also buttons in classic mode
        t_outlet *outlet_nc_acc; // l and r in classic mode (2 atoms)
        t_outlet *outlet_nc_stick; // first stick in classic mode
	t_outlet *outlet_classic_stick; // second stick in classic mode

} t_wiimote;

// the callback functions needs access to the object, but it cannot be
// passed as an argument. I use GArray because it can grow and shrink,
// contrary to a normal C array. The setup var is also global and keeps track off
// the number of connected wii remotes.
//
GArray *list;
int setup;

// prototype for the callback function, this is recommended by libcwiid 
// it should be easier to detect something is wrong if the callback function
// changes in newer versions of the library

cwiid_mesg_callback_t cwiid_callback;

// LED functions
void wiimote_setLED(t_wiimote *x, t_floatarg f);
void set_led_state(t_wiimote *x);
void wiimote_LED1(t_wiimote *x);
void wiimote_LED2(t_wiimote *x);
void wiimote_LED3(t_wiimote *x);
void wiimote_LED4(t_wiimote *x);

void wiimote_rumble(t_wiimote *x, t_floatarg f);

// report mode
void wiimote_setReportMode(t_wiimote *, t_floatarg f);
void wiimote_status(t_wiimote *x);
void wiimote_buttons(t_wiimote *x, t_floatarg f);
void wiimote_acceleration(t_wiimote *x, t_floatarg f);
void wiimote_IR(t_wiimote *x, t_floatarg f);
void wiimote_nunchuk(t_wiimote *x, t_floatarg f);
void wiimote_classic(t_wiimote *x, t_floatarg f);
void wiimote_extension(t_wiimote *x, t_floatarg f);

// output functions
void wiimote_button_report(t_wiimote *x, struct cwiid_btn_mesg *mesg);
void wiimote_acc_report(t_wiimote *x, struct cwiid_acc_mesg *mesg);
void wiimote_ir_report(t_wiimote *x, struct cwiid_ir_mesg *mesg);
void wiimote_nunchuk_report(t_wiimote *x, struct cwiid_nunchuk_mesg *mesg);
void wiimote_classic_report(t_wiimote *x, struct cwiid_classic_mesg *mesg);

// setup functions
void wiimote_discover(t_wiimote *x);
void wiimote_connect(t_wiimote *x, t_symbol *addr);
void wiimote_disconnect(t_wiimote *x);
void wiimote_free(t_wiimote *x);
void *wiimote_new(t_symbol *s, int argc, t_atom *argv);
void wiimote_setup(void);

// support functions
void wiimote_debug(t_wiimote *x);
